
import org.xml.sax.Attributes;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;


/**
 *  Main parser class for ROMIP dataset.
 *
 *  Note: 
 *     Despite the difference in the formats parser supports
 *       both ROMIP'2003 "narod.ru" collection as well as 
 *       ROMIP'2004 "legal documents" colelction and 
 *       training set for ROMIP'2004 web site classification track.
 *
 *  To customize parsing develop new class implementing PageHandler interface
 *  and pass it name as parameter on command line.
 *  See SimplePageHandler as an example.
 *
 *  Sample usage:
 *     java ROMIPParser SimplePageHandler
 */

public class ROMIPParser extends org.xml.sax.helpers.DefaultHandler
{
   public final static String VERSION = "1.0";

   private PageHandler handler;
   private String docID;
   private StringBuffer encodedContent;
   private String content;

   private String currentTag;

   private ROMIPParser()
   {
   };

   ROMIPParser(PageHandler h)
   {
      handler = h;
      encodedContent = new StringBuffer(10000);
   }

   public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
   {
      if (qName.equals("document")) {
         docID = "";
         encodedContent.setLength(0);
      }
      currentTag = qName;
   }

   public void characters(char[] ch, int start, int length)
   {
      if (currentTag != null) {
         if (currentTag.equals("docID")) {
            docID += new String(ch, start, length);
         }
         else if (currentTag.equals("content")) {
            encodedContent.append(ch, start, length);
         }
      }
   }

   public void endElement(String namespaceURI, String localName, String qName)
   {
      if (qName.equals("document")) {
         handler.handlePage(docID, Base64.decodeToString(encodedContent.toString()));
      }

      currentTag = null;
   }

   public static void main(String[] args)
   {
      ROMIPParser p;
      PageHandler handler = null;
      Class c = null;

      if (args.length < 1) {
         System.err.println("Official parser for ROMIP dataset (Version " + VERSION + ")\nUsage:\n  java ROMIPParser handler-class\n");
         return;
      }

      try {
         c = Class.forName(args[0]);

         if (c != null) {
            handler = (PageHandler) c.newInstance();
         }
      } catch (ClassNotFoundException e) {
         System.err.println("Failed to find handler class " + args[0]);
         return;
      } catch (Exception e) {
         System.err.println("Failed to instantiate handler class");
         e.printStackTrace();
         return;
      }

      try {
         p = new ROMIPParser(handler);

         SAXParser sax = SAXParserFactory.newInstance().newSAXParser();

         sax.parse(System.in, p);
      } catch (Exception e) {
         System.err.println("Failed to setup parsing environment or parse input stream\n" + e);
      }
   }
}
